
#include <sightglass.h>

#include <stdint.h>

#define ITERATIONS 10000

#define keccak_BLOCKBYTES 200
#define ROTL64(x, b) (uint32_t)(((x) << (b)) | ((x) >> (64 - (b))))

static void
keccak_core(uint64_t state[keccak_BLOCKBYTES / 8])
{
    uint64_t Aba;
    uint64_t Abe;
    uint64_t Abi;
    uint64_t Abo;
    uint64_t Abu;
    uint64_t Aga;
    uint64_t Age;
    uint64_t Agi;
    uint64_t Ago;
    uint64_t Agu;
    uint64_t Aka;
    uint64_t Ake;
    uint64_t Aki;
    uint64_t Ako;
    uint64_t Aku;
    uint64_t Ama;
    uint64_t Ame;
    uint64_t Ami;
    uint64_t Amo;
    uint64_t Amu;
    uint64_t Asa;
    uint64_t Ase;
    uint64_t Asi;
    uint64_t Aso;
    uint64_t Asu;
    uint64_t Ca;
    uint64_t Ce;
    uint64_t Ci;
    uint64_t Co;
    uint64_t Cu;
    uint64_t Da;
    uint64_t De;
    uint64_t Di;
    uint64_t Do;
    uint64_t Du;
    uint64_t Eba;
    uint64_t Ebe;
    uint64_t Ebi;
    uint64_t Ebo;
    uint64_t Ebu;
    uint64_t Ega;
    uint64_t Ege;
    uint64_t Egi;
    uint64_t Ego;
    uint64_t Egu;
    uint64_t Eka;
    uint64_t Eke;
    uint64_t Eki;
    uint64_t Eko;
    uint64_t Eku;
    uint64_t Ema;
    uint64_t Eme;
    uint64_t Emi;
    uint64_t Emo;
    uint64_t Emu;
    uint64_t Esa;
    uint64_t Ese;
    uint64_t Esi;
    uint64_t Eso;
    uint64_t Esu;

    Aba = state[0];
    Abe = state[1];
    Abi = state[2];
    Abo = state[3];
    Abu = state[4];
    Aga = state[5];
    Age = state[6];
    Agi = state[7];
    Ago = state[8];
    Agu = state[9];
    Aka = state[10];
    Ake = state[11];
    Aki = state[12];
    Ako = state[13];
    Aku = state[14];
    Ama = state[15];
    Ame = state[16];
    Ami = state[17];
    Amo = state[18];
    Amu = state[19];
    Asa = state[20];
    Ase = state[21];
    Asi = state[22];
    Aso = state[23];
    Asu = state[24];

    /* round 1 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x0000000000000001ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 2 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x0000000000008082ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 3 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x800000000000808AULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 4 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x8000000080008000ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 5 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x000000000000808BULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 6 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x0000000080000001ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 7 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x8000000080008081ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 8 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x8000000000008009ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 9 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x000000000000008AULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 10 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x0000000000000088ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 11 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x0000000080008009ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 12 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x000000008000000AULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 13 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x000000008000808BULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 14 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x800000000000008BULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 15 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x8000000000008089ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 16 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x8000000000008003ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 17 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x8000000000008002ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 18 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x8000000000000080ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 19 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x000000000000800AULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 20 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x800000008000000AULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 21 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x8000000080008081ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 22 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x8000000000008080ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);
    /* round 23 */
    Ca = Aba ^ Aga ^ Aka ^ Ama ^ Asa;
    Ce = Abe ^ Age ^ Ake ^ Ame ^ Ase;
    Ci = Abi ^ Agi ^ Aki ^ Ami ^ Asi;
    Co = Abo ^ Ago ^ Ako ^ Amo ^ Aso;
    Cu = Abu ^ Agu ^ Aku ^ Amu ^ Asu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Aba ^= Da;
    Ca = Aba;
    Age ^= De;
    Ce = ROTL64(Age, 44);
    Aki ^= Di;
    Ci = ROTL64(Aki, 43);
    Amo ^= Do;
    Co = ROTL64(Amo, 21);
    Asu ^= Du;
    Cu  = ROTL64(Asu, 14);
    Eba = Ca ^ ((~Ce) & Ci);
    Eba ^= 0x0000000080000001ULL;
    Ebe = Ce ^ ((~Ci) & Co);
    Ebi = Ci ^ ((~Co) & Cu);
    Ebo = Co ^ ((~Cu) & Ca);
    Ebu = Cu ^ ((~Ca) & Ce);
    Abo ^= Do;
    Ca = ROTL64(Abo, 28);
    Agu ^= Du;
    Ce = ROTL64(Agu, 20);
    Aka ^= Da;
    Ci = ROTL64(Aka, 3);
    Ame ^= De;
    Co = ROTL64(Ame, 45);
    Asi ^= Di;
    Cu  = ROTL64(Asi, 61);
    Ega = Ca ^ ((~Ce) & Ci);
    Ege = Ce ^ ((~Ci) & Co);
    Egi = Ci ^ ((~Co) & Cu);
    Ego = Co ^ ((~Cu) & Ca);
    Egu = Cu ^ ((~Ca) & Ce);
    Abe ^= De;
    Ca = ROTL64(Abe, 1);
    Agi ^= Di;
    Ce = ROTL64(Agi, 6);
    Ako ^= Do;
    Ci = ROTL64(Ako, 25);
    Amu ^= Du;
    Co = ROTL64(Amu, 8);
    Asa ^= Da;
    Cu  = ROTL64(Asa, 18);
    Eka = Ca ^ ((~Ce) & Ci);
    Eke = Ce ^ ((~Ci) & Co);
    Eki = Ci ^ ((~Co) & Cu);
    Eko = Co ^ ((~Cu) & Ca);
    Eku = Cu ^ ((~Ca) & Ce);
    Abu ^= Du;
    Ca = ROTL64(Abu, 27);
    Aga ^= Da;
    Ce = ROTL64(Aga, 36);
    Ake ^= De;
    Ci = ROTL64(Ake, 10);
    Ami ^= Di;
    Co = ROTL64(Ami, 15);
    Aso ^= Do;
    Cu  = ROTL64(Aso, 56);
    Ema = Ca ^ ((~Ce) & Ci);
    Eme = Ce ^ ((~Ci) & Co);
    Emi = Ci ^ ((~Co) & Cu);
    Emo = Co ^ ((~Cu) & Ca);
    Emu = Cu ^ ((~Ca) & Ce);
    Abi ^= Di;
    Ca = ROTL64(Abi, 62);
    Ago ^= Do;
    Ce = ROTL64(Ago, 55);
    Aku ^= Du;
    Ci = ROTL64(Aku, 39);
    Ama ^= Da;
    Co = ROTL64(Ama, 41);
    Ase ^= De;
    Cu  = ROTL64(Ase, 2);
    Esa = Ca ^ ((~Ce) & Ci);
    Ese = Ce ^ ((~Ci) & Co);
    Esi = Ci ^ ((~Co) & Cu);
    Eso = Co ^ ((~Cu) & Ca);
    Esu = Cu ^ ((~Ca) & Ce);
    /* round 24 */
    Ca = Eba ^ Ega ^ Eka ^ Ema ^ Esa;
    Ce = Ebe ^ Ege ^ Eke ^ Eme ^ Ese;
    Ci = Ebi ^ Egi ^ Eki ^ Emi ^ Esi;
    Co = Ebo ^ Ego ^ Eko ^ Emo ^ Eso;
    Cu = Ebu ^ Egu ^ Eku ^ Emu ^ Esu;
    Da = Cu ^ ROTL64(Ce, 1);
    De = Ca ^ ROTL64(Ci, 1);
    Di = Ce ^ ROTL64(Co, 1);
    Do = Ci ^ ROTL64(Cu, 1);
    Du = Co ^ ROTL64(Ca, 1);
    Eba ^= Da;
    Ca = Eba;
    Ege ^= De;
    Ce = ROTL64(Ege, 44);
    Eki ^= Di;
    Ci = ROTL64(Eki, 43);
    Emo ^= Do;
    Co = ROTL64(Emo, 21);
    Esu ^= Du;
    Cu  = ROTL64(Esu, 14);
    Aba = Ca ^ ((~Ce) & Ci);
    Aba ^= 0x8000000080008008ULL;
    Abe = Ce ^ ((~Ci) & Co);
    Abi = Ci ^ ((~Co) & Cu);
    Abo = Co ^ ((~Cu) & Ca);
    Abu = Cu ^ ((~Ca) & Ce);
    Ebo ^= Do;
    Ca = ROTL64(Ebo, 28);
    Egu ^= Du;
    Ce = ROTL64(Egu, 20);
    Eka ^= Da;
    Ci = ROTL64(Eka, 3);
    Eme ^= De;
    Co = ROTL64(Eme, 45);
    Esi ^= Di;
    Cu  = ROTL64(Esi, 61);
    Aga = Ca ^ ((~Ce) & Ci);
    Age = Ce ^ ((~Ci) & Co);
    Agi = Ci ^ ((~Co) & Cu);
    Ago = Co ^ ((~Cu) & Ca);
    Agu = Cu ^ ((~Ca) & Ce);
    Ebe ^= De;
    Ca = ROTL64(Ebe, 1);
    Egi ^= Di;
    Ce = ROTL64(Egi, 6);
    Eko ^= Do;
    Ci = ROTL64(Eko, 25);
    Emu ^= Du;
    Co = ROTL64(Emu, 8);
    Esa ^= Da;
    Cu  = ROTL64(Esa, 18);
    Aka = Ca ^ ((~Ce) & Ci);
    Ake = Ce ^ ((~Ci) & Co);
    Aki = Ci ^ ((~Co) & Cu);
    Ako = Co ^ ((~Cu) & Ca);
    Aku = Cu ^ ((~Ca) & Ce);
    Ebu ^= Du;
    Ca = ROTL64(Ebu, 27);
    Ega ^= Da;
    Ce = ROTL64(Ega, 36);
    Eke ^= De;
    Ci = ROTL64(Eke, 10);
    Emi ^= Di;
    Co = ROTL64(Emi, 15);
    Eso ^= Do;
    Cu  = ROTL64(Eso, 56);
    Ama = Ca ^ ((~Ce) & Ci);
    Ame = Ce ^ ((~Ci) & Co);
    Ami = Ci ^ ((~Co) & Cu);
    Amo = Co ^ ((~Cu) & Ca);
    Amu = Cu ^ ((~Ca) & Ce);
    Ebi ^= Di;
    Ca = ROTL64(Ebi, 62);
    Ego ^= Do;
    Ce = ROTL64(Ego, 55);
    Eku ^= Du;
    Ci = ROTL64(Eku, 39);
    Ema ^= Da;
    Co = ROTL64(Ema, 41);
    Ese ^= De;
    Cu  = ROTL64(Ese, 2);
    Asa = Ca ^ ((~Ce) & Ci);
    Ase = Ce ^ ((~Ci) & Co);
    Asi = Ci ^ ((~Co) & Cu);
    Aso = Co ^ ((~Cu) & Ca);
    Asu = Cu ^ ((~Ca) & Ce);

    state[0]  = Aba;
    state[1]  = Abe;
    state[2]  = Abi;
    state[3]  = Abo;
    state[4]  = Abu;
    state[5]  = Aga;
    state[6]  = Age;
    state[7]  = Agi;
    state[8]  = Ago;
    state[9]  = Agu;
    state[10] = Aka;
    state[11] = Ake;
    state[12] = Aki;
    state[13] = Ako;
    state[14] = Aku;
    state[15] = Ama;
    state[16] = Ame;
    state[17] = Ami;
    state[18] = Amo;
    state[19] = Amu;
    state[20] = Asa;
    state[21] = Ase;
    state[22] = Asi;
    state[23] = Aso;
    state[24] = Asu;
}

void
keccak_body(void *ctx)
{
    (void) ctx;

    uint64_t state[keccak_BLOCKBYTES / 8] = { 0 };
    BLACK_BOX(state);

    int i;
    for (i = 0; i < ITERATIONS; i++) {
        keccak_core(state);
    }
    BLACK_BOX(state[0]);
}
